/*

  ci_seq.c: Counter Increment Sequential.
  Copyright (C) 2001 Thomas Repantis <darkzero@otenet.gr>
  
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  
*/
 
/*

  This program increments a counter sequentially (without using threads and synchronization).

*/

#include <stdio.h>
#include <stdlib.h>   /* To use strtoul(). */
#include <errno.h>    /* To use errno. */
#include <sys/time.h> /* To use gettimeofday(). */

#ifdef MEASURE_TIME

#define CALLS_TO_GETTIMEOFDAY 2
#define MICROS_PER_SECOND 1000000

#endif


int main(int argc, char *argv[])
{
  
  long int counter = 0;
  unsigned long int target_value = 0;

#ifdef MEASURE_TIME
  /* In this array we keep the timeval structures returned by each call to gettimeofday(). */
  struct timeval times[CALLS_TO_GETTIMEOFDAY];
  unsigned long int time_elapsed = 0;
#endif

  printf("\n");
  printf("ci_seq: Sequential counter increment.\n");  
  printf("Copyright (C) 2001 Thomas Repantis <darkzero@otenet.gr>\n");
  printf("This program comes with ABSOLUTELY NO WARRANTY. This is free software, and you are welcome to redistribute it under certain conditions. ci_seq is licensed under the GNU General Public License. See the file COPYING for details.\n");
  printf("\n");

  if( argc != 2)
    {
      printf("Usage: ci_seq [Target value for the counter to reach.]\n");
      printf("For example: ci_seq 1000000\n");
      printf("\n");
      exit(1);
    }
  
  
  /* Convert the command line argument string to unsigned long integer. */
  
  errno = 0;
  target_value = strtoul(argv[1], (char**)NULL, 10);
  if( errno != 0 )
    {
      fprintf(stderr, "Error on the target value (%s) given.\n", argv[1]);
      fflush(stderr);
      exit(1);
    }
  
  fprintf(stdout, "The target value given is interpreted as: %lu\n", target_value);
  fflush(stdout);
  
  /* Increment the counter and measure time needed. */
  
  /* We avoid (the really rare) exception handling on gettimeofday(), not to lose in performance. */
  
#ifdef MEASURE_TIME
  gettimeofday(&times[0], NULL);
#endif
  
  for (counter = 0; counter < target_value; counter++)
    ;
  
#ifdef MEASURE_TIME
  gettimeofday(&times[1], NULL);
#endif  

  fprintf(stdout, "Counter has reached value: %lu\n", counter);
  fflush(stdout);

#ifdef MEASURE_TIME
  /* Calculate the seconds part of the time elapsed. */
  time_elapsed = (times[1].tv_sec - times[0].tv_sec)*MICROS_PER_SECOND;
  
  /* Calculate the microseconds part of the time elapsed. */
  time_elapsed += (times[1].tv_usec - times[0].tv_usec);
  
  /* Present the measured time in microsecond precision. */
  fprintf(stdout, "Measured time to increment the counter sequentially: %lu microseconds (%f seconds)\n", time_elapsed, time_elapsed/((double)MICROS_PER_SECOND));
  fflush(stdout);
#endif
  
}

	


