/*

  incr_barrier.c: Sense Reversing Centralized Incremental Barrier.
  Copyright (C) 2001 Thomas Repantis <darkzero@otenet.gr>
  
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  
*/
 
/*

  These functions implement the Sense Reversing Centralized Incremental Barrier.

*/

#include <stdio.h>
#include "machine.h"
#include "libsync.h"

#define UNFREE 0  /* Not-free, you cannot leave the barrier. */
#define FREE   1  /* Free, you can leave the barrier. */



void incr_barrier_init(struct incr_barrier *thr_incr_barrier, int processors)
{
  /* incr_barrier_init initializes the incr_barrier. */
  thr_incr_barrier->global_sense = UNFREE;
  thr_incr_barrier->processors = processors;
  thr_incr_barrier->counter = processors;
}

void incr_barrier(struct incr_barrier *thr_incr_barrier)
{
  /* incr_barrier makes the thread wait until all threads reach the incr_barrier pointed. */
 
  int local_sense;

  local_sense = thr_incr_barrier->global_sense;

  /* The if control must take place together with the fetch and add (atomically). */
  if( fetch_and_add(&thr_incr_barrier->counter, -1) != 1)   /* I am not the last one reaching the barrier. */
    {
      while( thr_incr_barrier->global_sense == local_sense )
	;
    }
  else  /* thr_incr_barrier->counter == 1 */   /* I am the last one reaching the barrier. */
    {
      thr_incr_barrier->counter = thr_incr_barrier->processors; 
      thr_incr_barrier->global_sense = ~thr_incr_barrier->global_sense; /* Bit negation or normal one (using !). */
    }

}

