/*

  libsync.h: Header file to use the libsync library.
  Copyright (C) 2001 Thomas Repantis <darkzero@otenet.gr>
  
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  
*/
 
/*

  This header file contains the declarations needed to use the libsync library 
  we implemented. 

*/

#define MAX_THR_NUM 100 /*The maximum number of threads being synchronized allowed. */
#define LONGPADS_PER_CACHE_LINE 8 /* The cache line of the SPARC is 32 Bytes long and a long 
				     is sizeof(long) = 4 Bytes long, therefore 32/4=8. */
#define INTPADS_PER_CACHE_LINE 8 /*  An int has the same size as a long. */
#define CHARPADS_PER_CACHE_LINE 32 /* A char is 1 Byte long, therefore 32/1=32. */
#define TIMEVALPADS_PER_CACHE_LINE 4 /*A timeval structure is sizeof(struct timeval) = 
				       8 Bytes long, therefore 32/8=4. */

/* We use padding to the size of the cache line to avoid false sharing. */


struct ttas_lock {
  char _pad1[CHARPADS_PER_CACHE_LINE];
  volatile char enter;
  char _pad2[CHARPADS_PER_CACHE_LINE - 1];
};


struct ticket_lock {  
  long _pad3[LONGPADS_PER_CACHE_LINE];
  volatile long next_ticket;
  long _pad4[LONGPADS_PER_CACHE_LINE - 1];
  volatile long now_serving;
  long _pad5[LONGPADS_PER_CACHE_LINE -1];
};


struct queue_lock {  
  long _pad6[LONGPADS_PER_CACHE_LINE];
  volatile long next_slot;
  long _pad7[LONGPADS_PER_CACHE_LINE - 1];
  volatile int queue[MAX_THR_NUM][INTPADS_PER_CACHE_LINE - 1]; /* A statically allocated array 
							      is enough (no need for calloc
							      and free). */
};


struct incr_barrier {  
  int _pad8[INTPADS_PER_CACHE_LINE];
  volatile int global_sense;
  int _pad9[INTPADS_PER_CACHE_LINE - 1];
  volatile long counter;
  long _pad10[LONGPADS_PER_CACHE_LINE - 1];
  volatile int processors;
  int _pad11[INTPADS_PER_CACHE_LINE - 1];
};


extern void ttas_init( struct ttas_lock * );
extern void ttas_lock( struct ttas_lock * );
extern void ttas_unlock( struct ttas_lock * );

extern void ticket_init( struct ticket_lock * );
extern void ticket_lock( struct ticket_lock * );
extern void ticket_unlock( struct ticket_lock * );

extern void queue_init( struct queue_lock * );
extern int queue_lock( struct queue_lock * );
extern void queue_unlock( struct queue_lock *, int );

extern void incr_barrier_init( struct incr_barrier *, int );
extern void incr_barrier( struct incr_barrier * );
