/*

  queue_lock.c: Array Based Queue Lock.
  Copyright (C) 2001 Thomas Repantis <darkzero@otenet.gr>
  
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  
*/
 
/*

  These functions implement the Array Based Queue Lock.

*/

#include <stdio.h>
#include "machine.h"
#include "libsync.h"

#define UNFREE 0  /* Not-free, you cannot enter. */
#define FREE   1  /* Free, you can enter. */



void queue_init(struct queue_lock *thr_queue_lock_ptr)
{
  /* queue_init initializes the queue_lock. */
  int i;

  thr_queue_lock_ptr->next_slot = 0;
  
  /* We initialize the queue array. */
  for (i=1; i<MAX_THR_NUM; i++)
    thr_queue_lock_ptr->queue[i][0] = UNFREE;
  
  thr_queue_lock_ptr->queue[0][0] = FREE; /* So that the first one can enter. */
}

int queue_lock(struct queue_lock *thr_queue_lock_ptr)
{
  /*queue_lock acquires the queue_lock pointed to it by a thread. */

  int cur_position = 0;  /* Each thread has its own current position. */
  cur_position = fetch_and_add(&(thr_queue_lock_ptr->next_slot), 1);
  cur_position = cur_position % MAX_THR_NUM;
  
  while( thr_queue_lock_ptr->queue[cur_position][0] != FREE )
    ;
  
  return cur_position;
}

void queue_unlock(struct queue_lock *thr_queue_lock_ptr, int cur_position)
{
  /* queue_unlock releases the queue_lock pointed to it by a thread. */

  thr_queue_lock_ptr->queue[cur_position][0] = UNFREE;
  thr_queue_lock_ptr->queue[ ((cur_position + 1) % MAX_THR_NUM) ][0] = FREE;
  /* We use the modulo, so that the counter is circular (pointing to the beginning of the 
     array again, after reaching its end, instead of pointing out of the array's borders). */
}


