/*

  ttas_lock.c: Test and Test and Set Lock.
  Copyright (C) 2001 Thomas Repantis <darkzero@otenet.gr>
  
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  
*/
 
/*

  These functions implement the Test and Test and Set Lock.

*/

#include <stdio.h>
#include "machine.h"
#include "libsync.h"

#define FREE   0  /* Free, you can enter. */
#define UNFREE 1  /* Not-free, you cannot enter. */


void ttas_init(struct ttas_lock *thr_ttas_lock_ptr)
{
  /* ttas_init initializes the ttas_lock. */
  
  thr_ttas_lock_ptr->enter = FREE;
}



void ttas_lock(struct ttas_lock *thr_ttas_lock_ptr)
{
  /* ttas_lock acquires the ttas_lock pointed to it by a thread. */

 still_unfree:
  while( thr_ttas_lock_ptr->enter != FREE )
    ;

  if( test_and_set(&thr_ttas_lock_ptr->enter) == FREE )
    ;
  else
    goto still_unfree; /* Sorry! */
}



void ttas_unlock(struct ttas_lock *thr_ttas_lock_ptr)
{
  /* ttas_unlock releases the ttas_lock pointed to it by a thread. */

  thr_ttas_lock_ptr->enter = FREE;
}
